S <- matrix(c(1, .5, .5, .5, .5,
               .5, 1, .5, .5, .5,
               .5, .5, 1, .5, .5,
               .5, .5, .5, 1, .5,
               .5, .5, .5, .5, 1), 5, 5)
N <- 500

ll.FA1 <- function(theta, S, N) {
	q <- nrow(S)
	L <- matrix(rep(theta, q), nrow=1, ncol=q)
	Phi <- matrix(1, 1, 1)
	Delta <- diag(1, q, q) - diag(rep(theta*theta, q), q, q)
	LPL <- t(L) %*% Phi %*% L
	Sigma <- LPL + Delta
	SigmaInverse <- solve(Sigma)
	SigmaS <- SigmaInverse %*% S
	l <- (500/2)*(log(det(Sigma)) + (sum(diag(SigmaS))))
}

# start the factor loading ("theta") at 0.6
theta <- 0.6
system.time( result <- nlm(f=ll.FA1,p=theta,hessian=TRUE,S=S,N=N) )
print(result$estimate)
SE <- sqrt(diag(solve(result$hessian)))
print(SE)

# watch it fail from a start of 0.7 (which is actually closer!)
theta <- 0.7
system.time( result <- nlm(f=ll.FA1,p=theta,hessian=TRUE,print.level=2,S=S,N=N) )

# a stepsize limit of 0.2 will bring it in:
theta <- 0.7
system.time( result <- nlm(f=ll.FA1,p=theta,hessian=TRUE,print.level=2,
		stepmax=0.2,S=S,N=N) )


