S <- matrix(c(1, .5, .5, .5, .5,
               .5, 1, .5, .5, .5,
               .5, .5, 1, .5, .5,
               .5, .5, .5, 1, .5,
               .5, .5, .5, .5, 1), 5, 5)
N <- 500

ll.FA1 <- function(theta, S, N) {
	q <- nrow(S)
	L <- matrix(rep(theta, q), nrow=1, ncol=q)
	Phi <- matrix(1, 1, 1)
	Delta <- diag(1, q, q) - diag(rep(theta*theta, q), q, q)
	LPL <- t(L) %*% Phi %*% L
	Sigma <- LPL + Delta
	SigmaInverse <- solve(Sigma)
	SigmaS <- SigmaInverse %*% S
	l <- (500/2)*(log(det(Sigma)) + (sum(diag(SigmaS))))
}

theta <- 0.6
system.time( result <- nlm(f=ll.FA1,p=theta,hessian=TRUE,S=S,N=N) )
SE <- sqrt(diag(solve(result$hessian)))

# this is to make my plots for the slides:
# these are the plots for the loading-as-(NR)parameter
thetalist <- seq(0.1, 0.9, 0.01)
lllist <- rep(0,length(thetalist))
for (i in 1:length(thetalist)) {
	lllist[i] <- ll.FA1(thetalist[i], S, N)
}
quartz(width=10, height=10)
par(cex.axis=2, cex.lab=2, mar=(c(5, 8, 4, 2)+0.1), lab=c(9,6,7))
plot(thetalist, lllist, type="l", xlab="loading", ylab="-loglikelihood", col="blue")

thetalist <- seq(0.65, 0.75, 0.001)
lllist <- rep(0,length(thetalist))
for (i in 1:length(thetalist)) {
	lllist[i] <- ll.FA1(thetalist[i], S, N)
}
quartz(width=10, height=10)
par(cex.axis=2, cex.lab=2, mar=(c(5, 8, 4, 2)+0.1), lab=c(9,6,7))
plot(thetalist, lllist, type="l", xlab="loading", ylab="-loglikelihood", col="blue")

#########################################

ll.FA1J <- function(theta, S, N) {
	q <- nrow(S)
	Phi <- matrix(1, 1, 1)
	Delta <- diag(rep(theta, q), q, q)
	L <- matrix(rep(sqrt(1-theta), q), nrow=1, ncol=q)
	LPL <- t(L) %*% Phi %*% L
	Sigma <- LPL + Delta
	SigmaInverse <- solve(Sigma)
	SigmaS <- SigmaInverse %*% S
	l <- (500/2)*(log(det(Sigma)) + (sum(diag(SigmaS))))
}

# here "theta" is the unique variance, as per Jennrich & Robinson
theta <- 0.5
system.time( result <- nlm(f=ll.FA1J,p=theta,hessian=TRUE,S=S,N=N) )
SE <- sqrt(diag(solve(result$hessian)))

thetalist <- seq(0.1, 0.9, 0.01)
lllist <- rep(0,length(thetalist))
for (i in 1:length(thetalist)) {
	lllist[i] <- ll.FA1J(thetalist[i], S, N)
}
quartz(width=10, height=10)
par(cex.axis=2, cex.lab=2, mar=(c(5, 8, 4, 2)+0.1), lab=c(9,6,7))
plot(thetalist, lllist, type="l", xlab="unique variance", ylab="-loglikelihood", col="red")

thetalist <- seq(0.4, 0.6, 0.001)
lllist <- rep(0,length(thetalist))
for (i in 1:length(thetalist)) {
	lllist[i] <- ll.FA1J(thetalist[i], S, N)
}
quartz(width=10, height=10)
par(cex.axis=2, cex.lab=2, mar=(c(5, 8, 4, 2)+0.1), lab=c(9,6,7))
plot(thetalist, lllist, type="l", xlab="unique variance", ylab="-loglikelihood", col="red")

#########################################

ll.FA2 <- function(theta, S, N) {
	q <- nrow(S)
	L <- matrix(rep(theta[1], q), nrow=1, ncol=q)
	Phi <- matrix(1, 1, 1)
	Delta <- diag(theta[2], q, q)
	LPL <- t(L) %*% Phi %*% L
	Sigma <- LPL + Delta
	SigmaInverse <- solve(Sigma)
	SigmaS <- SigmaInverse %*% S
	l <- (500/2)*(log(det(Sigma)) + (sum(diag(SigmaS))))
}

# the limit on stepsize fixes it:
theta <- c(0.6, 0.8)
system.time( result <- nlm(f=ll.FA2,p=theta,hessian=TRUE,print.level=2,
		stepmax=0.2,S=S,N=N) )


# for contour plot:
theta1list <- seq(0.5, 0.9, length=100)
theta2list <- seq(0.3, 0.7, length=100)
z <- matrix(0, nrow=100, ncol=100)
for (i in 1:100) {
	for (j in 1:100) {
		theta <- c(theta1list[i], theta2list[j])
		z[i,j] <- ll.FA2(theta, S, N)
	}
 }
quartz(width=10, height=10)
par(cex.axis=2, cex.lab=2, cex.main=2, mar=(c(5, 8, 4, 2)+0.1), lab=c(7,7,7))
contour(theta1list, theta2list, z, labcex=1.5, col="magenta",
    main = "-likelihood Contours, Equal Loading Factor Analysis",
    xlab = "Loading",
	ylab = "Unique Variance")

# for a second contour plot, both modes:
theta1list <- seq(-0.9, 0.9, length=100)
theta2list <- seq(0.3, 0.7, length=100)
z <- matrix(0, nrow=100, ncol=100)
for (i in 1:100) {
	for (j in 1:100) {
		theta <- c(theta1list[i], theta2list[j])
		z[i,j] <- ll.FA2(theta, S, N)
	}
 }
quartz(width=10, height=10)
par(cex.axis=2, cex.lab=2, cex.main=2, mar=(c(5, 8, 4, 2)+0.1), lab=c(7,7,7))
contour(theta1list, theta2list, z, labcex=1.5, col="magenta",
	levels = c(840, 880, 920, 960, 1000, 1500, 2000),
    main = "-likelihood Contours, Equal Loading Factor Analysis",
    xlab = "Loading",
	ylab = "Unique Variance")
	
#########################################

# these data are the first "congeneric tests" example
# from Joreskog (1971) and the Lisrel manual
S <- matrix(c(25.0704, 12.4363, 11.7257, 20.7510,
               12.4363, 28.2012, 9.2281, 11.9732,
               11.7257, 9.2281, 22.7390, 12.0692,
               20.7510, 11.9732, 12.0692, 21.8707), 4, 4)
N <- 126

q <- nrow(S)
R <- matrix(1, 1, 1)

# starting values:
theta <- c(4.5, 3, 3, 4.5, 4, 21, 15, 1)
beta <- matrix(theta[1:q], nrow=1, ncol=q)
tau2 <- diag(theta[(q+1):(2*q)], q, q)

G2list <- rep(0,200)

# EM loop
for (i in 1:200) {
	inverseterm <- solve((t(beta) %*% beta) + tau2)
	delta <- inverseterm %*% t(beta)
	CapDelta <- R - ((beta %*% inverseterm) %*% t(beta))
	inverseterm2 <- solve((t(delta) %*% S %*% delta) + CapDelta)

	beta <- inverseterm2 %*% t(S %*% delta)
	tau2 <- diag(diag(S - (S %*% delta %*% beta)))
#	print(beta)

	Sigma <- (t(beta) %*% R %*% beta) + tau2
	SigmaS <- solve(Sigma) %*% S
	G2 <- (N-1) * (log(det(Sigma)/det(S)) + sum(diag(SigmaS)) - nrow(S))
#	print(c(i, G2))
	G2list[i] <- G2
}

quartz(width=10, height=10)
par(cex.axis=2, cex.lab=2, mar=(c(5, 8, 4, 2)+0.1), lab=c(9,6,7))
plot((1:200), G2list, type="l", xlab="EM Cycle", ylab="G-squared", col="orangered")

quartz(width=10, height=10)
par(cex.axis=2, cex.lab=2, mar=(c(5, 8, 4, 2)+0.1), lab=c(9,6,7))
plot((100:200), G2list[100:200], type="l", xlab="EM Cycle", ylab="G-squared", col="orangered")

